/*
    This file is part of KDiary.

    Copyright (c) 2004 cjacker huang <jzhuang@redflag-linux.com>
    Copyright (c) 2005 Bram Schoenmakers <bramschoenmakers@kde.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with kdiary program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    As a special exception, permission is given to link kdiary program
    with any edition of Qt, and distribute the resulting executable,
    without including the source code for Qt in the source distribution.
*/

#include "kdiary.h"
#include "kdiary.moc"
#include "ui/kdiaryconfigdialog.h"
#include "ui/finddialog.h"
#include <stdio.h>
#include <qregexp.h>
#include "tkcoloractions.h"
#include <kcalendarsystem.h>

KDiary *kdiaryapp;

KDiary::~KDiary () {
}


KDiary::KDiary () {
  config = kapp->config ();
  kdiaryapp=this;
  cdlg = new KDiaryConfigDialog ();
  cdlg->hide();
  appDir = KGlobal::dirs ()->saveLocation ("data", "kdiary");
  QDir::setCurrent (appDir.path ());
  KIconLoader kiconloader ("kdiary");
  QPixmap first = KGlobal::iconLoader ()->loadIcon ("lastentry", KIcon::Toolbar);
  QIconSet firstI (first);
  QPixmap previous = KGlobal::iconLoader ()->loadIcon ("previousentry", KIcon::Toolbar);
  QIconSet previousI (previous);
  QPixmap yearback = KGlobal::iconLoader ()->loadIcon ("yearback", KIcon::Toolbar);
  QIconSet yearbackI (yearback);
  QPixmap monthback = KGlobal::iconLoader ()->loadIcon ("monthback", KIcon::Toolbar);
  QIconSet monthbackI (monthback);
  QPixmap dayback = KGlobal::iconLoader ()->loadIcon ("dayback", KIcon::Toolbar);
  QIconSet daybackI (dayback);
  QPixmap dayforward = kiconloader.loadIcon ("dayforward", KIcon::Toolbar);
  QIconSet forwardI (dayforward);
  QPixmap today = kiconloader.loadIcon ("today", KIcon::Toolbar);
  QIconSet todayI (today);
  QPixmap nextentry = kiconloader.loadIcon ("nextentry", KIcon::Toolbar);
  QIconSet nextentryI (nextentry);
  QPixmap newestentry = kiconloader.loadIcon ("newestentry", KIcon::Toolbar);
  QIconSet newestentryI (newestentry);
  QPixmap monthforward = kiconloader.loadIcon ("monthforward", KIcon::Toolbar);
  QIconSet monthfowardI (monthforward);
  QPixmap yearforward = kiconloader.loadIcon ("yearforward", KIcon::Toolbar);
  QIconSet yearforwardI (yearforward);


    //    KStdAction::spelling( this, SLOT (slotSpelling()),actionCollection());

  (void) new KAction (i18n ("Insert Date"),KShortcut( CTRL + Key_D), this, SLOT (slotInsertDate ()),
  actionCollection (), "insertDate");
  (void) new KAction (i18n ("Insert Time"),KShortcut( CTRL + Key_T), this, SLOT (slotInsertTime ()),
  actionCollection (), "insertTime");
  (void) new KAction (i18n ("Insert Date And Time"), KShortcut( CTRL + SHIFT + Key_D), this,
  SLOT (slotInsertDateAndTime ()), actionCollection (),
  "insertDateAndTime");
  (void) new KAction (i18n ("Insert &Image"), KShortcut( CTRL + SHIFT + Key_I), this,
  SLOT (slotInsertImage ()), actionCollection (),
  "insertImage");

  (void) new KAction (i18n ("KDiary &Configuration"),
  QIconSet (KGlobal::iconLoader ()->
      loadIcon ("configure", KIcon::Small)),
  KShortcut (), this, SLOT (slotOpenSetupDlg ()),
  actionCollection (), "openSetupDlg");

  statusBar ()->hide();
  menuBar ()->hide();
  QFrame *mainpage = new QFrame((QWidget*)this);
  QVBoxLayout *vLayout= new QVBoxLayout(mainpage);
  QHBoxLayout *mainLayout = new QHBoxLayout(mainpage );
  mListView=new QListView(mainpage);
  mListView->setFixedWidth(fontMetrics().width("    xxxxxxxx    ")+20);
  mListView->addColumn( i18n("Entries") ,fontMetrics().width("    xxxxxxxx    ")+16);
  mListView->setTreeStepSize(20);
  mTextEdit = new KTextEdit (mainpage);
  mTextEdit->setTextFormat (Qt::RichText);

  loadList();


  mTextEdit->mimeSourceFactory ()->setFilePath (appDir.path ());
  connect (mListView, SIGNAL( doubleClicked ( QListViewItem *)), this, SLOT(loadEntrytoText(QListViewItem *)));
  connect (mTextEdit,SIGNAL(currentFontChanged ( const QFont & )),this,SLOT(slotFontChanged(const QFont &)));
  connect (mTextEdit,SIGNAL(currentColorChanged ( const QColor & )),this,SLOT(slotColorChanged(const QColor &)));
  if (config->readBoolEntry ("wrap")) {
    mTextEdit->setWordWrap (QTextEdit::FixedColumnWidth);
  }

  actionFormatFontFamily = new KFontAction( i18n( "Font Family" ), 0,
                                            actionCollection(), "format_fontfamily" );
  connect( actionFormatFontFamily, SIGNAL( activated( const QString & ) ),
           this, SLOT( setFont( const QString & ) ) );
  QFont f;
  if (config->readEntry ("font")) {
    f.fromString (config->readEntry ("font"));
  } else {
    f = QApplication::font ();
  }
  actionFormatFontFamily->setFont (f.family ());

  actionFormatFontSize=new KFontSizeAction(i18n("Font Size"),0,actionCollection(),"format_fontsize");
  connect (actionFormatFontSize, SIGNAL (fontSizeChanged(int)), this, SLOT (setFontSize(int)));
  actionFormatFontSize->setEnabled( true );
  actionFormatFontSize->setFontSize(f.pointSize());

  actionFormatBold=new KToggleAction(i18n("&Bold"),"text_bold",CTRL + Key_B,this,SLOT(slotBold()),
                                     actionCollection(),"format_bold");
  actionFormatBold->setEnabled(true);

  actionFormatItalic = new KToggleAction( i18n( "&Italic" ), "text_italic", CTRL + Key_I,
                                          this, SLOT( slotItalic() ),
                                          actionCollection(), "format_italic" );
  actionFormatUnderline = new KToggleAction( i18n( "&Underline" ), "text_under", CTRL + Key_U,
                                             this, SLOT( slotUnderline() ),
                                             actionCollection(), "format_underline" );
  actionFormatAlignLeft = new KToggleAction( i18n( "Align &Left" ), "text_left", CTRL + Key_L,
                                             this, SLOT( slotLeft() ),
                                             actionCollection(), "format_alignleft" );
  actionFormatAlignLeft->setExclusiveGroup( "align" );
  actionFormatAlignLeft->setChecked( TRUE );
  actionFormatAlignCenter = new KToggleAction( i18n( "Align &Center" ), "text_center", CTRL
      + ALT + Key_C,
      this, SLOT( slotCenter() ),
      actionCollection(), "format_aligncenter" );
  actionFormatAlignCenter->setExclusiveGroup( "align" );
  actionFormatAlignRight = new KToggleAction( i18n( "Align &Right" ), "text_right", CTRL +
      ALT + Key_R,
      this, SLOT( slotRight() ),
      actionCollection(), "format_alignright" );
  actionFormatAlignRight->setExclusiveGroup( "align" );
  actionFormatAlignBlock = new KToggleAction( i18n( "Align &Block" ), "text_block", CTRL +
      Key_J,
      this, SLOT( slotBlock() ),
      actionCollection(), "format_alignblock" );
  actionFormatAlignBlock->setExclusiveGroup( "align" );

  actionFormatColor=new TKSelectColorAction( i18n( "Text Color..." ), TKSelectColorAction::TextColor,
                                             this, SLOT( slotColor() ),actionCollection(), "format_color",true );
  actionFormatColor->setCurrentColor (QColor (config->readEntry ("fgColor", "#000000")));

  actionInsertDT = new KAction( i18n( "Insert Date and Time" ), "clock", CTRL +
      SHIFT+Key_D,
      this, SLOT( slotInsertDateAndTime() ),
      actionCollection(), "insertdt" );
  actionInsertIMG = new KAction( i18n( "Insert Image" ), "background", CTRL +
      SHIFT+Key_I,
      this, SLOT( slotInsertImage() ),
      actionCollection(), "insertimg" );
  menuBar()->hide();
  statusBar()->hide();

  KToolBar * newToolBar= new KToolBar(mainpage);
  newToolBar->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
  //newToolBar->setFixedWidth(600);
  QDate qdate;
  currDate = qdate;
  todayD = qdate.currentDate ();
  newToolBar->insertButton (today, 5, SIGNAL (clicked (int)), this,
                            SLOT (slotToday ()), true, i18n("Go to Today"));
  newToolBar->insertLineSeparator();
  actionFormatFontFamily->plug(newToolBar);
  actionFormatFontSize->plug(newToolBar);
  actionFormatBold->plug(newToolBar);
  actionFormatItalic->plug(newToolBar);
  actionFormatUnderline->plug(newToolBar);
  actionFormatColor->plug(newToolBar);
  actionFormatAlignLeft->plug(newToolBar);
  actionFormatAlignCenter->plug(newToolBar);
  actionFormatAlignRight->plug(newToolBar);
  actionFormatAlignBlock->plug(newToolBar);
  newToolBar->insertLineSeparator();
  actionInsertDT->plug(newToolBar);
  actionInsertIMG->plug(newToolBar);
  newToolBar->insertLineSeparator();
  dateL = new QLabel (KGlobal::locale()->formatDate( qdate ), newToolBar);
  slotToday ();

  newToolBar->setFixedHeight(28);
  mainLayout->addWidget(mListView);
  mainLayout->addWidget(mTextEdit);
  vLayout->addWidget(newToolBar);


  vLayout->addLayout(mainLayout);
  setCentralWidget (mainpage);
    //actionInsertImage= new KAction(i18n(" Insert Image..."),"frame_image",0, this, SLOT(slotInsertImage()), actionCollection(), "insert_image");
    //actionInsertHtml= new KAction(i18n(" Insert HTML code..."), "view_text",0,this, SLOT(slotInsertHTML), actionCollection(), "insert_html");

//    createGUI (locate ("data", "kdiary/kdiaryui.rc"));
  slotToday ();
  setAutoSaveSettings ();

}

bool KDiary::printDialog () {
  return true;
}

void KDiary::loadList(){
  mListView->clear();
  QDir d(appDir.path ());
  const QFileInfoList* roots=d.entryInfoList();
  QPtrListIterator<QFileInfo> i(*roots);
  QFileInfo* fi;
  while ((fi=*i)){
    ++i;
    QString fileName=fi->fileName();
    if (fileName.find(QRegExp("\\d\\d\\d\\d\\d\\d\\d\\d"))!=-1){
      QString year=fileName.left(4);
      QString month=fileName.mid(4,2);
      QString day=fileName.right(2);

      bool found=false;
      QListViewItem* yearLV=(QListViewItem*)mListView->firstChild();
      if (yearLV){
        while ( yearLV && !found){
          if ( (yearLV->text(0)).compare(year)==0 ){
            found=true;
          }else{
            yearLV=(QListViewItem*)yearLV->nextSibling();
          }
        }
      }
      if (!found){
        new QListViewItem(mListView,year);
      }

      yearLV=(QListViewItem*)mListView->firstChild();
      found=false;
      QListViewItem* monthLV;
      while ( yearLV && !found){
        if ( (yearLV->text(0)).compare(year)==0 ){
          monthLV=(QListViewItem*)yearLV->firstChild();
          while ( monthLV && !found){
            if ( (monthLV->text(0)).compare(month)==0 ){
              found=true;

            }else{
              monthLV=(QListViewItem*)monthLV->nextSibling();
            }
          }
          if (!found){
            new QListViewItem(yearLV,month);
          }

          found=true;
        }else{
          yearLV=(QListViewItem*)yearLV->nextSibling();
        }
      }

      monthLV=(QListViewItem*)yearLV->firstChild();
      found=false;
      while ( monthLV && !found){
        if ( (monthLV->text(0)).compare(month)==0 ){
          QListViewItem* dayLV=(QListViewItem*)monthLV->firstChild();
          while ( dayLV && !found){
            if ( (dayLV->text(0)).compare(day)==0 ){
              found=true;

            }else{
              dayLV=(QListViewItem*)dayLV->nextSibling();
            }
          }
          if (!found){
            new QListViewItem(monthLV,day);
          }

          found=true;
        }else{
          monthLV=(QListViewItem*)monthLV->nextSibling();
        }
      }
    }
  }
}

bool KDiary::print () {
  return true;
}


void KDiary::selectAll () {
  mTextEdit->selectAll ();
}

void KDiary::deselect () {
  mTextEdit->selectAll (false);
}

void KDiary::undo () {
  mTextEdit->undo ();
}

void KDiary::redo () {
  mTextEdit->redo ();
}

void KDiary::cut () {
  mTextEdit->cut ();
}


void KDiary::copy () {
  mTextEdit->copy ();
}

void KDiary::paste () {
  mTextEdit->paste ();
}


void KDiary::toggleToolBar () {
}

void KDiary::toggleStatusBar () {
}

void KDiary::saveOptions () {
  mTextEdit->selectAll (false);
}

KDiary::saveResult KDiary::save () {
  QString dirname;
  getDate_Str (currDate, &dirname);
  QDir d (appDir.path ());
  if (!d.exists (d.filePath (dirname))) {
    d.mkdir (dirname);
  }
  d.setPath (d.filePath (dirname));
  QFile file (d.filePath ("entry"));  // Write the text to a file
  if (file.open (IO_WriteOnly)) {
    QTextStream ts (&file);
    QString
        sText (mTextEdit->text ());
    ts << sText;
    mTextEdit->setModified (FALSE);
    file.close ();
    loadList();

/*
    int index = 0;
    QRegExp imgRE ("<img src=(.+) >");
    imgRE.setMinimal (TRUE);
    QStrList slImg;
    while ((index = sText.find (imgRE, index)) != -1) {
    slImg.append (imgRE.cap (1));
    index++;
  }
    QString dirname;
    getDate_Str (currDate, &dirname);
    QDir dirEntry (appDir.path () + "/" + dirname);
    dirEntry.setFilter (QDir::Files);
    const QFileInfoList *Flist = dirEntry.entryInfoList ();
    QFileInfoListIterator itFI (*Flist);
    QStrListIterator itIMG (slImg);
    QFileInfo *fi;
    QString img;
    QFileInfoList deleteFlist;
    int found;
    while ((fi = itFI.current ())) {
    found = 0;
    itIMG.toFirst ();
    while (img = itIMG.current ()) {
    if (img.find (fi->fileName ().data ()) != -1) {
    found = 1;
  }
    ++itIMG;
  }
    if (found == 0 && QString::compare (fi->fileName ().data (), "entry") != 0) {
    dirEntry.remove (fi->fileName ().data ());
  }
    ++itFI;
  }
*/
    return SAVE_OK;
  }
  return SAVE_CANCEL;
}

void KDiary::loadEntrytoText(QListViewItem * cur_item)
{
  if (cur_item->depth()==2)
  {
    int year =  (cur_item->parent()->parent()->text(0)).toInt();
    int month =  (cur_item->parent()->text(0)).toInt();
    int day = cur_item->text(0).toInt();
    QDate this_date(year, month, day);
    loadEntry(this_date);
  }

}

bool KDiary::loadEntry (QDate qdate) {
  if (entryChanged () == SAVE_CANCEL)
    return false;
  QString dirname;
  currDate = qdate;
  getDate_Str (qdate, &dirname);
  QDir d (appDir.path ());
  QFile f (d.filePath (dirname) + "/entry");

  config->setGroup ("Editor");
  QFont font;
  if (config->readEntry ("font")) {
    font.fromString (config->readEntry ("font"));
  } else {
    font = QApplication::font ();
  }
  if (f.exists ()) {
    if (f.open (IO_ReadOnly)) {
      QTextStream t (&f);
      mTextEdit->setColor (QColor (config->readEntry ("fgColor", "#000000")));
      mTextEdit->
          setPaper (QBrush
          (QColor (config->readEntry ("bgColor", "#ffffff"))));
      mTextEdit->setText (t.read ());
      f.close ();
      emit loadEntry();
    }
  } else {
    QTime time;
    QString todayStr( KGlobal::locale()->formatDate( todayD ) );
    QTextStream t (&todayStr, IO_ReadOnly);

    mTextEdit->setText ("");
    mTextEdit->setCurrentFont (font);
    mTextEdit->setColor (QColor (config->readEntry ("fgColor", "#000000")));
    mTextEdit->setPaper (QBrush (QColor (config->readEntry ("bgColor", "#ffffff"))));
    mTextEdit->setTextFormat (Qt::RichText);
    mTextEdit->insert (t.read ());
  }

  mTextEdit->setModified (false);
  return true;
}

void KDiary::getDate_Str (QDate qdate, QString * string) {
  int date;
  QString date_str;
  int day = qdate.day ();
  int month = qdate.month ();
  int year = qdate.year ();
  year = year * 10000;
  month = month * 100;
  date = year + month + day;
  string->setNum (date);
}

void KDiary::getDate_QD (QDate * qdate, QString * string) {
  QString year = string->left (4);
  QString month = string->mid (4, 2);
  QString day = string->right (2);
  qdate->setYMD (atoi (year), atoi (month), atoi (day));
}

void KDiary::slotLastEntry () {
  QDate qdate;
  int lastEntryDist = 999999;
  QString todayC;
  getDate_Str (currDate, &todayC);
  QString lastEntry (todayC);
  int todayI = atoi (todayC);
  QDir d = QDir::current ();
  d.setFilter (QDir::Dirs);
  const QFileInfoList *Flist = d.entryInfoList ();
  QFileInfoListIterator it (*Flist);
  QFileInfo *fi;
  while ((fi = it.current ())) {
    if (todayI - atoi (fi->fileName ().data ()) < lastEntryDist) {
      int dist = todayI - atoi (fi->fileName ().data ());
      if (dist > 0) {
        lastEntryDist = dist;
        lastEntry = fi->fileName ().data ();
      }
    }
    ++it;
  }
  getDate_QD (&qdate, &lastEntry);
  if (currDate != qdate) {
    setDate (qdate);
  } else {
  }
}

void KDiary::slotNextEntry () {

  QDate qdate;
  int nextEntryDist = 999999;
  QString todayC;
  getDate_Str (currDate, &todayC);
  QString nextEntry (todayC);
  int todayI = atoi (todayC);

  QDir d = QDir::current ();
  d.setFilter (QDir::Dirs);
  const QFileInfoList *Flist = d.entryInfoList ();
  QFileInfoListIterator it (*Flist);
  QFileInfo *fi;
  while ((fi = it.current ())) {
    if (atoi (fi->fileName ().data ()) - todayI < nextEntryDist) {
      int dist = atoi (fi->fileName ().data ()) - todayI;
      if (dist > 0) {
        nextEntryDist = dist;
        nextEntry = fi->fileName ().data ();
      }
    }
    ++it;
  }
  getDate_QD (&qdate, &nextEntry);
  if (currDate != qdate) {
    setDate (qdate);
  } else {
  }

}

KDiary::saveResult KDiary::entryChanged () {
  int
      query;

  if (mTextEdit->isModified ()) {
    if (!config->readBoolEntry ("autosave")) {
      query = KMessageBox::warningYesNoCancel (this,
                                               i18n
                                                   ("The current Document has been modified.\nWould you like to save it?"));
    } else {
      query = KMessageBox::Yes;
    }
    switch (query) {
      case KMessageBox::Yes:  //yes
        if (save () == SAVE_CANCEL)
          return SAVE_CANCEL;
        if (mTextEdit->isModified ()) {
          query = KMessageBox::warningContinueCancel (this,
              i18n
                  ("Could not save the document.\nDiscard it and continue?"),
              QString::null,
              i18n ("&Discard"));
          if (query == KMessageBox::Cancel)
            return SAVE_CANCEL;
        }
        break;
        case KMessageBox::Cancel: //cancel
          return SAVE_CANCEL;
      case KMessageBox::No:
        return SAVE_NO;
    }
  }
  return SAVE_OK;
}

void KDiary::slotToday () {
  setDate (todayD.currentDate ());
}

void KDiary::slotFirstEntry () {
  QDir d = QDir::current ();
  d.setFilter (QDir::Dirs);
  QStringList Flist = d.entryList ();
  QString firstEntry (*Flist.at (2));
  if (!firstEntry.contains (QRegExp ("\\d{8}")) || !firstEntry)
    return;
  QDate qdate;
  getDate_QD (&qdate, &firstEntry);
  setDate (qdate);

}

void KDiary::slotYearBack () {
  setDate (currDate.addYears (-1));
}

void KDiary::slotMonthBack () {
  setDate (currDate.addMonths (-1));
}

void KDiary::slotDayBack () {
  setDate (currDate.addDays (-1));
}

void KDiary::slotDayForward () {
  setDate (currDate.addDays (1));
}

void KDiary::slotMonthForward () {
  setDate (currDate.addMonths (1));
}

void KDiary::slotYearForward () {
  setDate (currDate.addYears (1));
}

void KDiary::slotNewestEntry () {
  QDir d = QDir::current ();
  d.setFilter (QDir::Dirs);
  const QFileInfoList *Flist = d.entryInfoList ();
  if (!Flist)
    return;
  QFileInfoListIterator it (*Flist);
  QFileInfo *fi = it.toLast ();
  QString lastEntry = fi->fileName ().data ();
  QDate qdate;
  if (!lastEntry.contains (QRegExp ("\\d{8}")))
    return;
  getDate_QD (&qdate, &lastEntry);
  setDate (qdate);
}

bool KDiary::queryClose () {
  if (entryChanged () == SAVE_CANCEL)
    return false;
  kapp->quit ();
  return true;
}


/** No descriptions */
void KDiary::setDate (QDate date) {

  if (loadEntry (date)) {
    currDate = date;
    dateL->setText( KGlobal::locale()->formatDate( currDate ) );
  }

}

/** No descriptions */
void KDiary::slotInsertDate () {
  mTextEdit->insert(KGlobal::locale()->formatDate( todayD ) );
}

/** No descriptions */
void KDiary::slotInsertTime () {
  QTime time;
  time.currentTime ();
  mTextEdit->insert ((time.currentTime ()).toString ());
}

/** No descriptions */
void KDiary::slotInsertDateAndTime () {
  QTime time;
  mTextEdit->insert (todayD.toString ("yyyy M.dd dddd") + " - " + // FIXME use locale
      (time.currentTime ()).toString ());
}


/** No descriptions */
void KDiary::slotOpenSetupDlg () {

  cdlg->show ();
}

/** No descriptions */
KConfig *
    KDiary::getConfig () {
  return config;
    }

    /** No descriptions */
    void KDiary::slotBold () {
      mTextEdit->setBold (actionFormatBold->isChecked());
    }

    /** No descriptions */
    void KDiary::slotUnderline () {
      mTextEdit->setUnderline (actionFormatUnderline->isChecked ());
    }

    /** No descriptions */
    void KDiary::slotItalic () {
      mTextEdit->setItalic (actionFormatItalic->isChecked ());
    }

    /** No descriptions */
    void KDiary::setFont (const QString &font) {
      int fontSize=actionFormatFontSize->fontSize();
      mTextEdit->setCurrentFont (QFont(font).family());
      mTextEdit->setPointSize(fontSize);
      mTextEdit->setFocus();
    }

    /** sets font and colors */
    void KDiary::applyConfig () {
      config->setGroup ("Editor");
      QFont font;
      font.fromString (config->readEntry ("font"));
      mTextEdit->setCurrentFont (font);

      actionFormatFontFamily->setFont(font.family());
      actionFormatFontSize->setFontSize(font.pointSize());
      actionFormatColor->setCurrentColor(QColor(config->readEntry ("fgColor", "#000000")));
      mTextEdit->setColor (QColor (config->readEntry ("fgColor", "#000000")));
      mTextEdit->setPaper (QBrush (QColor (config->readEntry ("bgColor", "#ffffff"))));
      if (config->readBoolEntry ("wrap")) {
        mTextEdit->setWordWrap (QTextEdit::FixedColumnWidth);
        mTextEdit->setWrapColumnOrWidth (config->readNumEntry ("wrapAt"));
      } else {
        mTextEdit->setWordWrap (QTextEdit::WidgetWidth);
      }
    }

    /** No descriptions */
    void KDiary::slotPrint () {
      KPrinter printer;
      printer.setFullPage (true);
      if (printer.setup (this)) {
        QPainter p (&printer);
        // Check that there is a valid device to print to.
        if (!p.device ())
          return;
        QPaintDeviceMetrics metrics (p.device ());
        int dpix = metrics.logicalDpiX ();
        int dpiy = metrics.logicalDpiY ();
        const int margin = 72;  // pt
        QRect body (margin * dpix / 72, margin * dpiy / 72,
                    metrics.width () - margin * dpix / 72 * 2,
                    metrics.height () - margin * dpiy / 72 * 2);
        QFont font ("times", 10);
        QSimpleRichText richText (mTextEdit->text (), font, mTextEdit->context (),
                                  mTextEdit->styleSheet (),
                                  mTextEdit->mimeSourceFactory (), body.height ());
        richText.setWidth (&p, body.width ());
        QRect view (body);
        int page = 1;
        do {
          richText.draw (&p, body.left (), body.top (), view, colorGroup ());
          view.moveBy (0, body.height ());
          p.translate (0, -body.height ());
          p.setFont (font);
          p.drawText (view.right () -
              p.fontMetrics ().width (QString::number (page)),
          view.bottom () + p.fontMetrics ().ascent () + 5,
          QString::number (page));
          if (view.top () >= richText.height ())
            break;
          printer.newPage ();
          page++;
        } while (true);
      }
    }

    /** No descriptions */
    void KDiary::slotFind () {
      FindDialog *fd = new FindDialog (appDir.path (),this);
      fd->show ();
    }

    /** No descriptions */
    void KDiary::slotSpelling () {}

    /** inserts image */
    void KDiary::slotInsertImage () {

      KURL url =
          KFileDialog::getImageOpenURL (QDir::homeDirPath (), this,
                                        i18n ("Select Image"));
      if (!url.isValid())
        return;
      if (!url.isLocalFile ()) {
        KMessageBox::sorry (this,
                            i18n ("Currently only local images are possible."));
        return;
      }


      QString filename = url.path ();
      QFile in (filename);
      QFileInfo inFileInfo (filename);
      filename = inFileInfo.fileName ();
      QDir d (appDir.path ());
      QString dirname;
      getDate_Str (currDate, &dirname);
      if (inFileInfo.filePath () != d.filePath (dirname) + "/" + filename) {
        QFile out (d.filePath (dirname) + "/" + filename);
        if (!d.exists (d.filePath (dirname))) {
          d.mkdir (dirname);
        }
        in.open (IO_ReadOnly);
        out.open (IO_WriteOnly);
        int size = inFileInfo.size ();
        char *buffer = new char[size];
        in.readBlock (buffer, size);
        out.writeBlock (buffer, size);
        in.close ();
        out.close ();
      }

      insertHTML("<img src=\"" + d.filePath (dirname) + "/" + filename + "\">");
    }

    /** text will be leftaligned */
    void KDiary::slotLeft () {
      if ( actionFormatAlignLeft->isChecked() ) {
        mTextEdit->setAlignment (Qt::AlignLeft);
      } else
        actionFormatAlignLeft->setChecked( true );

    }

    /** text will be rightaligned */
    void KDiary::slotRight () {
      if ( actionFormatAlignRight->isChecked() ) {
        mTextEdit->setAlignment (Qt::AlignRight);
      } else
        actionFormatAlignRight->setChecked( true );
    }

    /** text will be centered */
    void KDiary::slotCenter () {
      if ( actionFormatAlignCenter->isChecked() ) {
        mTextEdit->setAlignment (Qt::AlignCenter);
      } else
        actionFormatAlignCenter->setChecked( true );
    }

    void KDiary::slotBlock() {
      if ( actionFormatAlignBlock->isChecked() ) {
        mTextEdit->setAlignment (Qt::AlignJustify);
      } else
        actionFormatAlignBlock->setChecked( true );
    }

    void KDiary::insert (QString string) {
      mTextEdit->insert (string);
    }

    /** returns the date of the loaded entry */
    QDate KDiary::today () {
      return currDate;
    }

    /** No descriptions */
    void KDiary::select (int para, int index, int length) {
      mTextEdit->setSelection (para, index, para, index + length);
    }


    QString KDiary::entryDirPath() {
      QString dirname;
      getDate_Str (currDate, &dirname);
      QDir d (appDir.path ());
      return QString(d.filePath (dirname));
    }

    void KDiary::insertHTML(QString HTMLtext) {
      int para,index;
      QString textCp=mTextEdit->text();
      mTextEdit->getCursorPosition(&para,&index);
      int pos=0,j=-1;
      while(j++<para) {
        pos=textCp.find("<p",pos);
        pos++;
      }
      while (QChar(textCp[(pos++)])!=">")
        ;

      j=0;
      int i=0;

      while(j<index) {
        if (QChar(textCp[pos+j])=="<") {
          while(QChar(textCp[(pos++)+j])!=">") {
            i++;
          }
          i=0;
        } else {
          j++;
        }
      }

      textCp.insert(pos+j,HTMLtext);
      mTextEdit->setText(textCp);
      mTextEdit->setCursorPosition(para,index+1);
      mTextEdit->setModified(true);
    }

    void KDiary::setFontSize(int size) {
      mTextEdit->setPointSize(size);
      mTextEdit->setFocus();
    }

    void KDiary::slotColor() {
    //color of icon does not change if color is selected from color chooser
      actionFormatColor->setCurrentColor(actionFormatColor->color() );
      mTextEdit->setColor(actionFormatColor->color());
      mTextEdit->setFocus();
    }

    /** called by text (QTextEdit) */
    void KDiary::slotFontChanged(const QFont & f) {
      actionFormatFontSize->setFontSize(f.pointSize());
      actionFormatFontFamily->setFont (f.family ());
    }

    /** called by text (QTextEdit) */
    void KDiary::slotColorChanged(const QColor & c) {
      actionFormatColor->setCurrentColor(c);
    }
