/*
    This file is part of KDiary.

    Copyright (c) 2004 cjacker huang <jzhuang@redflag-linux.com>
    Copyright (c) 2005 Bram Schoenmakers <bramschoenmakers@kde.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with kdiary program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    As a special exception, permission is given to link kdiary program
    with any edition of Qt, and distribute the resulting executable,
    without including the source code for Qt in the source distribution.
*/

#ifndef KDIARY_H
#define KDIARY_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fstream>
#include <stdio.h>
#include <stdlib.h>

#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qsimplerichtext.h>

#include <kaction.h>
#include <kapplication.h>
#include <kdatepicker.h>
#include <kfiledialog.h>
#include <dcopclient.h>
#include <kfontcombo.h>
#include <kiconloader.h>
#include <klistview.h>
#include <klocale.h>
#include <kmainwindow.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kprinter.h>
#include <kstandarddirs.h>
#include <kstatusbar.h>
#include <ktextedit.h>

class TKSelectColorAction;
class KFontSizeAction;
class KDiaryConfigDialog;

/** KDiary is the base class of the project */
class KDiary : public KMainWindow {
    Q_OBJECT
public:
    /** construtor */
    KDiary();
    /** destructor */
    ~KDiary();
    /** No descriptions */
    KConfig* getConfig();
    /** sets font and colors */
    void applyConfig();
    void insert(QString string);
    /** returns the date of the loaded entry */
    QDate today();
    /** selects specified text */
    void select(int para,int index,int length);

    //void getCursorPosition(int* para,int* index){ text->getCursorPosition(para,index); };
    KDiaryConfigDialog *preferencesBox() {
        return cdlg;
    }
    QString getText() {
        return mTextEdit->text();
    }
    void setText(QString _text) {
        mTextEdit->setText(_text);
    };

    /** inserts text into the entry */
    void insertHTML(QString HTMLtext);

private:
    KDiaryConfigDialog *cdlg;
     QListView* mListView;
    KTextEdit *mTextEdit;
    KDatePicker* dateW;
    KFontCombo* fontCombo;
    KFontSizeAction* actionFormatFontSize;
    QLabel* dateL;
    QDate currDate;
    QDate todayD;
    QDir appDir;
    KConfig *config;
    //KAction *actionInsertImage;
    //KAction *actionInsertHtml;
    KFontAction *actionFormatFontFamily;
    KToggleAction *actionFormatBold;
    KToggleAction *actionFormatItalic;
    KToggleAction *actionFormatUnderline;
    TKSelectColorAction *actionFormatColor;
    KToggleAction *actionFormatAlignLeft;
    KToggleAction *actionFormatAlignCenter;
    KToggleAction *actionFormatAlignRight;
    KToggleAction *actionFormatAlignBlock;
    KAction *actionInsertDT;
    KAction *actionInsertIMG;

    enum saveResult{ SAVE_OK, SAVE_CANCEL,SAVE_NO, SAVE_RETRY, SAVE_ERROR };

    void getDate_Str(QDate qdate,QString* string);
    void getDate_QD(QDate *qdate,QString* string);
    saveResult entryChanged();

public slots:
    /** opens a KDatepicker dialog */
    bool print();
    bool printDialog();
    void selectAll();
    void deselect();
    void undo();
    void redo();
    void cut();
    void copy();
    void paste();
    void toggleToolBar();
    void toggleStatusBar();
    void saveOptions();
    void loadList();
    void loadEntrytoText(QListViewItem *);
    saveResult save();
    bool loadEntry(QDate);
    void slotLastEntry();
    void slotNextEntry();
    void slotToday();
    void slotFirstEntry();
    void slotYearBack();
    void slotMonthBack();
    void slotDayBack();
    void slotDayForward();
    void slotMonthForward();
    void slotYearForward();
    void slotNewestEntry();
    bool queryClose();
    void setFontSize(int size);
    /** No descriptions */
    void setDate(QDate date);
    /** No descriptions */
    void setFont(const QString &font);

    /** used by plugins to determine the directory */
    QString entryDirPath();

signals:
    void loadEntry();
    void saveEntry();

private slots: // Private slots
    /** No descriptions */
    void slotInsertDate();
    /** No descriptions */
    void slotInsertDateAndTime();
    /** No descriptions */
    void slotInsertTime();
    /** No descriptions */
    /** No descriptions */
    void slotOpenSetupDlg();
    /** No descriptions */
    void slotBold();
    /** No descriptions */
    void slotItalic();
    /** No descriptions */
    void slotUnderline();
    /** No descriptions */
    void slotPrint();
    /** No descriptions */
    void slotSpelling();
    /** No descriptions */
    void slotFind();
    /** No descriptions */
    void slotInsertImage();
    /** No descriptions */
    void slotLeft();
    /** No descriptions */
    void slotCenter();
    /** No descriptions */
    void slotRight();
    /** No descriptions */
    void slotBlock();
    /** sets the text color, called by tkselectcoloraction */
    void slotColor();
    void slotFontChanged(const QFont & f);
    void slotColorChanged(const QColor & f);
};

#endif
